@@ -7,21 +7,23 @@
 
 
 from .NotebookConfig import create_config
-from .Server import Server, mapping
-from .Resource import Resource, GpuType
-from .Executable import ProgramType
-from .Workload import Workload, Progress
+from .Server import Server
+from .Resource import Resource
+from .Workload import Workload
 from .ContainerRegistry import ContainerRegistry
+from .EnvironmentVariables import EnvironmentVariables
 
 import optumi_core as optumi
+from optumi_core.exceptions import OptumiException
 
-import os, datetime, json
-from enum import Enum
+import os, datetime, json, re
 from typing import Union
 
 
 class Container:
     def __init__(self, image: str, registry: ContainerRegistry = None):
+        if not bool(re.match("^[a-zA-Z0-9][a-zA-Z0-9/_.-]+$", image)):
+            raise OptumiException("Invalid container name '" + image + "'")
         self._image = image
         self._registry = registry
 
@@ -31,21 +33,36 @@ class Container:
     def launch(
         self,
         wait=True,
-        progress=Progress.SUMMARY,
-        resource=Server(size="Standard_NC4as_T4_v3"),
+        progress="summary",
+        env: Union[EnvironmentVariables, list] = [],
+        args=[],
+        resource=None,
         notifications=None,
     ):
+        if resource is None:
+            resource = Server(size="Standard_NC4as_T4_v3")
+
+        if progress != None and not progress in Workload.progress:
+            raise OptumiException(
+                "Unexpected progress '"
+                + progress
+                + "', expected on of "
+                + str(Workload.progress)
+            )
+
         # Start with blank config
         nb_config = create_config()
 
         # Plug in program type
-        nb_config["programType"] = ProgramType.DOCKER_CONTAINER.value
+        nb_config["programType"] = "docker container"
 
         # Register any unsaved container registries with the controller
         # Plug in container registries
 
+        nb_config["integrations"] = []
+
         if self._registry:
-            nb_config["integrations"] = [
+            nb_config["integrations"] += [
                 {
                     "name": self._registry.name,
                     "enabled": True,
@@ -53,18 +70,36 @@ class Container:
                 }
             ]
 
+        # Register any unsaved environment variables with the controller
+        # Plug in environment variables
+        if type(env) is EnvironmentVariables:
+            nb_config["integrations"] += [
+                {
+                    "name": env.name,
+                    "enabled": True,
+                    "integrationType": "environment variable",
+                }
+            ]
+        else:
+            nb_config["integrations"] += [
+                {
+                    "name": e.name,
+                    "enabled": True,
+                    "integrationType": "environment variable",
+                }
+                for e in env
+            ]
+
         # Plug in resource requirements
         if type(resource) is Server:
-            nb_config["machineAssortment"] = [
-                mapping[resource.provider.value] + ":" + resource.size
-            ]
+            nb_config["machineAssortment"] = [resource.provider + ":" + resource.size]
         elif type(resource) is Resource:
             nb_config["machineAssortment"] = []
             if type(resource.gpu) is bool:
                 nb_config["graphics"]["cores"] = [1 if resource.gpu else -1, -1, -1]
-            elif type(resource.gpu) is GpuType:
+            elif type(resource.gpu) is str:
                 nb_config["graphics"]["cores"] = [1, -1, -1]
-                nb_config["graphics"]["boardType"] = resource.gpu.value
+                nb_config["graphics"]["boardType"] = resource.gpu
 
             nb_config["graphics"]["memoryPerCard"] = resource.memory_per_card
 
@@ -77,15 +112,21 @@ class Container:
                 "packageReadySMSEnabled": False,
             }
 
-        container_name = (
-            self._registry.url + "/" + self._image if self._registry else self._image
-        )
+        container_name = self._image
 
         setup = json.loads(
             optumi.core.setup_notebook(
                 container_name,
                 self.__utcnow(),
-                {"path": container_name, "content": container_name},
+                {
+                    "path": container_name,
+                    "content": json.dumps(
+                        {
+                            "containerName": container_name,
+                            "args": args,
+                        }
+                    ),
+                },
                 json.dumps(nb_config),
             ).text
         )
@@ -116,7 +157,9 @@ class Container:
 
         # print(launch_status)
 
-        module_uuid = launch_status["modules"][0]
+        module_uuid = (
+            launch_status["modules"][0] if "modules" in launch_status else None
+        )
 
         workload = Workload(
             container_name,
