@@ -8,9 +8,9 @@
 from .NotebookConfig import create_config
 from .Packages import Packages
 from .LocalStorage import LocalStorage
-from .Server import Server, mapping
-from .Resource import Resource, GpuType
-from .Workload import Progress, Workload
+from .Server import Server
+from .Resource import Resource
+from .Workload import Workload
 from .EnvironmentVariables import EnvironmentVariables
 
 import optumi_core as optumi
@@ -20,20 +20,21 @@ from optumi_core.exceptions import (
 )
 
 import os, datetime, json
-from enum import Enum
 from typing import Union
 
 
-class ProgramType(Enum):
-    PYTHON_NOTEBOOK = "python notebook"
-    PYTHON_SCRIPT = "python script"
-    DOCKER_CONTAINER = "docker container"
-    UNKNOWN = "unknown"
-
-
 class Executable:
-    def __init__(self, path: str, program_type=ProgramType.UNKNOWN):
+    program_types = ["python notebook", "python script", "docker container", "unknown"]
+
+    def __init__(self, path: str, program_type="unknown"):
         self._path = optumi.utils.normalize_path(path)
+        if not program_type in Executable.program_types:
+            raise OptumiException(
+                "Unexpected program type '"
+                + program_type
+                + "', expected on of "
+                + str(program_types)
+            )
         self._program_type = program_type
 
     def __utcnow(self):
@@ -42,14 +43,25 @@ class Executable:
     def launch(
         self,
         wait=True,
-        progress=Progress.SUMMARY,
+        progress="summary",
         packages=Packages(),
         files=LocalStorage(),
         env: Union[EnvironmentVariables, list] = [],
-        resource=Server(size="Standard_NC4as_T4_v3"),
+        resource=None,
         notifications=None,
         interactive=False,
     ):
+        if resource is None:
+            resource = Server(size="Standard_NC4as_T4_v3")
+
+        if progress != None and not progress in Workload.progress:
+            raise OptumiException(
+                "Unexpected progress '"
+                + progress
+                + "', expected on of "
+                + str(Workload.progress)
+            )
+
         # Start with blank config
         nb_config = create_config()
 
@@ -57,7 +69,7 @@ class Executable:
         nb_config["interactive"] = interactive
 
         # Plug in program type
-        nb_config["programType"] = self._program_type.value
+        nb_config["programType"] = self._program_type
 
         # Plug in packages
         nb_config["upload"]["requirements"] = "\n".join(packages)
@@ -94,16 +106,14 @@ class Executable:
 
         # Plug in resource requirements
         if type(resource) is Server:
-            nb_config["machineAssortment"] = [
-                mapping[resource.provider.value] + ":" + resource.size
-            ]
+            nb_config["machineAssortment"] = [resource.provider + ":" + resource.size]
         elif type(resource) is Resource:
             nb_config["machineAssortment"] = []
             if type(resource.gpu) is bool:
                 nb_config["graphics"]["cores"] = [1 if resource.gpu else -1, -1, -1]
-            elif type(resource.gpu) is GpuType:
+            elif type(resource.gpu) is str:
                 nb_config["graphics"]["cores"] = [1, -1, -1]
-                nb_config["graphics"]["boardType"] = resource.gpu.value
+                nb_config["graphics"]["boardType"] = resource.gpu
 
             nb_config["graphics"]["memoryPerCard"] = resource.memory_per_card
 
