@@ -18,7 +18,7 @@ class Log:
 
     def download(self, path: str = None):
         f_name = optumi.utils.normalize_path(
-            self._name.split("/")[-1] + ".log" if path is None else path
+            self._name.split("/")[-1] + ".log" if path is None else path, False
         )
         with open(f_name, "w+") as f:
             f.write(
