@@ -9,18 +9,11 @@ import optumi_core as optumi
 from optumi_core.exceptions import OptumiException
 
 import json, time
-from enum import Enum
-
-
-class Status(Enum):
-    ACQUIRING = "Acquiring"
-    CONFIGURING = "Configuring"
-    BUSY = "Busy"
-    IDLE = "Idle"
-    RELEASING = "Releasing"
 
 
 class Machine:
+    status_values = ["Acquiring", "Configuring", "Busy", "Idle", "Releasing"]
+
     def __init__(
         self,
         uuid: str,
@@ -32,7 +25,7 @@ class Machine:
         state: str = None,
     ):
         self._uuid = uuid
-        self._size = size.replace("Azure:", "AZ:")
+        self._size = size
         self._dns_name = dns_name
         self._rate = "$" + str(round(rate, 2)) + "/hr"
         self._promo = promo
@@ -141,18 +134,18 @@ class Machine:
     @property
     def status(self):
         if self._state in ["requisition requested", "requisition in progress"]:
-            return Status.ACQUIRING
+            return "Acquiring"
         elif self._state in ["requisition completed"]:
-            return Status.CONFIGURING
+            return "Configuring"
         elif self._state in ["setup completed"]:
-            return Status.BUSY if self._app != None else Status.IDLE
+            return "Busy" if self._app != None else "Idle"
         elif self._state in [
             "teardown requested",
             "sequestration requested",
             "sequestration in progress",
             "sequestration completed",
         ]:
-            return Status.RELEASING
+            return "Releasing"
         else:
             return ""
 
