@@ -9,19 +9,31 @@ import optumi_core as optumi
 
 import json
 
-from .Machine import Machine, Status
+from .Machine import Machine
+
+from optumi_core.exceptions import (
+    OptumiException,
+)
 
 
 class Machines:
     @classmethod
-    def list(cls, status: Status = None):
+    def list(cls, status: str = None):
+        if status != None and not status in Machine.status_values:
+            raise OptumiException(
+                "Unexpected machine status '"
+                + status
+                + "', expected on of "
+                + str(Machine.status_values)
+            )
+
         machines = []
 
         response = json.loads(optumi.core.get_machines().text)
 
         for machine in response["machines"]:
             machine = Machine(*Machine.reconstruct(machine))
-            if machine.is_visible():
+            if (status is None and machine.is_visible()) or (machine.status == status):
                 machines.append(machine)
 
         return machines
