@@ -10,7 +10,7 @@ from .Executable import *
 
 class Notebook(Executable):
     def __init__(self, path: str):
-        super().__init__(path, ProgramType.PYTHON_NOTEBOOK)
+        super().__init__(path, "python notebook")
 
     def __str__(self):
         return super().__str__()
