@@ -23,7 +23,8 @@ class Program:
                 extension, "-" + str(self._run_num) + extension
             )
             if path is None
-            else path
+            else path,
+            False,
         )
         with open(f_name, "w+") as f:
             f.write(self._program)
