@@ -5,23 +5,23 @@
 ## To receive a copy of the licensing terms please write to contact@optumi.com or visit us at https://www.optumi.com.
 ##
 
-from enum import Enum
+from .api import graphics_card_types
 from typing import Union
 
-
-class GpuType(Enum):
-    UNSPECIFIED = "U"
-    NVIDIA_K80 = "K80"
-    NVIDIA_M60 = "M60"
-    NVIDIA_P100 = "P100"
-    NVIDIA_P40 = "P40"
-    NVIDIA_V100 = "V100"
-    NVIDIA_T4 = "T4"
-    NVIDIA_A100 = "A100"
+from optumi_core.exceptions import (
+    OptumiException,
+)
 
 
 class Resource:
-    def __init__(self, gpu: Union[bool, GpuType] = True, memory_per_card=0):
+    def __init__(self, gpu: Union[bool, str] = True, memory_per_card=0):
+        if type(gpu) is str and not gpu.lower() in [x.lower() for x in graphics_card_types()]:
+            raise OptumiException(
+                "Unexpected GPU type '"
+                + gpu
+                + "', expected on of "
+                + str(graphics_card_types())
+            )
         self._gpu = gpu
         self._memory_per_card = memory_per_card
 
