@@ -10,7 +10,7 @@ from .Executable import *
 
 class Script(Executable):
     def __init__(self, path: str):
-        super().__init__(path, ProgramType.PYTHON_SCRIPT)
+        super().__init__(path, "python script")
 
     def __str__(self):
         return super().__str__()
