@@ -31,7 +31,7 @@ class Summary:
 
     def download(self, path: str = None):
         f_name = optumi.utils.normalize_path(
-            self._name.split("/")[-1] + ".summary" if path is None else path
+            self._name.split("/")[-1] + ".summary" if path is None else path, False
         )
         with open(f_name, "w+") as f:
             f.write(
