@@ -19,209 +19,171 @@ from .utils import *
 import json, time, os, re
 from sys import exit
 
-from enum import Enum
-
-
-class Status(Enum):
-    QUEUED = "queued"
-    LAUNCHING = "launching"
-    RUNNING = "running"
-    COMPLETED = "completed"
-
-
-class Progress(Enum):
-    SILENT = "silent"
-    # STATUS = "status"
-    SUMMARY = "summary"
-    DETAIL = "detail"
-
 
 class Workload:
-    def __init__(
-        self,
-        path: str,
-        program: str,
-        workload_uuid: str,
-        module_uuid: str,
-        nb_config: dict,
-        run_num: str,
-        initializing_lines: list = [],
-        preparing_lines: list = [],
-        running_lines: list = [],
-        input_files: list = [],
-        update_lines: list = [],
-        output: list = [],
-        num_patches: int = 0,
-        output_files: list = [],
-        machine: Machine = None,
-        token: str = None,
-    ):
+    status_values = ["queued", "launching", "running", "completed"]
+    progress = ["silent", "summary", "detail"]  # add "status" later
+
+    def __init__(self, path, program, workload_uuid, module_uuid, config, run_num):
         self._path = path
         self._program = program
 
         self._workload_uuid = workload_uuid
         self._module_uuid = module_uuid
 
-        self._nb_config = nb_config
+        self._nb_config = config
 
         self._run_num = run_num
 
-        self._initializing_lines = initializing_lines.copy()
-        self._preparing_lines = preparing_lines.copy()
-        self._running_lines = running_lines.copy()
-        self._input_files = CloudStorage(input_files.copy())
+        self._initializing_lines = []
+        self._preparing_lines = []
+        self._running_lines = []
+        self._input_files = CloudStorage([])
 
-        self._update_lines = update_lines.copy()
-        self._output = output.copy()
-        self._last_patches = num_patches
+        self._update_lines = []
+        self._output = []
+        self._monitoring = []
+        self._last_patches = 0
 
-        self._output_files = CloudStorage(output_files.copy())
+        self._output_files = CloudStorage([])
 
-        self._machine = machine
-        self._token = token
+        self._machine = None
+        self._token = None
 
         self._last_refresh = time.time()
 
-    @classmethod
-    def reconstruct(cls, app_map):
-        # Reconstruct the app
-        initializing = []
-        for i in range(len(app_map["initializing"])):
-            initializing.append(
-                (app_map["initializing"][i], app_map["initializingmod"][i])
-            )
+    def _handle_workload_update(self, workload):
+        updates_to_print = []
 
-        preparing = []
-        for i in range(len(app_map["preparing"])):
-            preparing.append((app_map["preparing"][i], app_map["preparingmod"][i]))
+        if "uuid" in workload and workload["uuid"] != None:
+            self._workload_uuid = workload["uuid"]
 
-        running = []
-        for i in range(len(app_map["running"])):
-            running.append((app_map["running"][i], app_map["runningmod"][i]))
+        if "initializing" in workload and workload["initializing"] != None:
+            for i in range(len(workload["initializing"])):
+                self._initializing_lines.append(
+                    (workload["initializing"][i], workload["initializingmod"][i])
+                )
 
-        input_files = CloudStorage([])
-        if "files" in app_map and app_map["files"] != None:
-            input_files = CloudStorage(
+        if "preparing" in workload and workload["preparing"] != None:
+            for i in range(len(workload["preparing"])):
+                self._preparing_lines.append(
+                    (workload["preparing"][i], workload["preparingmod"][i])
+                )
+                updates_to_print.append(self._preparing_lines[-1])
+
+        if "running" in workload and workload["running"] != None:
+            for i in range(len(workload["running"])):
+                self._running_lines.append(
+                    (workload["running"][i], workload["runningmod"][i])
+                )
+                updates_to_print.append(self._running_lines[-1])
+
+        if "files" in workload and workload["files"] != None:
+            self._input_files = CloudStorage(
                 [
                     CloudFile(
-                        app_map["files"][i],
+                        workload["files"][i],
                         [
                             CloudFileVersion(
-                                app_map["files"][i],
-                                app_map["hashes"][i],
-                                int(app_map["filessize"][i]),
-                                app_map["filescrt"][i],
-                                app_map["filesmod"][i],
+                                workload["files"][i],
+                                workload["hashes"][i],
+                                int(workload["filessize"][i]),
+                                workload["filescrt"][i],
+                                workload["filesmod"][i],
                             )
                         ],
                     )
-                    for i in range(len(app_map["files"]))
+                    for i in range(len(workload["files"]))
                 ]
             )
 
-        module_uuid = None
-        output = []
-        updates = []
-        output_files = CloudStorage([])
-        num_patches = 0
-        program = None
-        machine = None
-        token = None
+        if "modules" in workload:
+            for module in workload["modules"]:
+                self._handle_module_update(module)
 
-        # Add modules
-        for module in app_map["modules"]:
-            module_uuid = module["uuid"]
+        return updates_to_print
 
-            if "output" in module:
-                for i in range(len(module["output"])):
-                    output.append((module["output"][i], module["outputmod"][i]))
+    def _handle_module_update(self, module):
+        output_to_print = []
 
-            if "updates" in module:
-                for i in range(len(module["updates"])):
-                    updates.append((module["updates"][i], module["updatesmod"][i]))
+        if "uuid" in module and module["uuid"] != None:
+            self._module_uuid = module["uuid"]
 
-            if "files" in module:
-                output_files = CloudStorage(
-                    [
-                        CloudFile(
-                            module["files"][i],
-                            [
-                                CloudFileVersion(
-                                    module["files"][i],
-                                    module["hashes"][i],
-                                    int(module["filessize"][i]),
-                                    module["filescrt"][i],
-                                    module["filesmod"][i],
-                                )
-                            ],
-                        )
-                        for i in range(len(module["files"]))
-                    ]
+        if "output" in module and module["output"] != None:
+            for i in range(len(module["output"])):
+                self._output.append((module["output"][i], module["outputmod"][i]))
+                output_to_print.append(self._output[-1])
+
+        if "updates" in module and module["updates"] != None:
+            for i in range(len(module["updates"])):
+                self._update_lines.append(
+                    (module["updates"][i], module["updatesmod"][i])
                 )
 
-            # monitoring = []
-            # if 'monitoring' in module:
-            #     for i in range(len(module['monitoring'])):
-            #         monitoring.append((module['monitoring'][i], module['monitoringmod'][i]))
+        if "files" in module and module["files"] != None:
+            self._output_files = CloudStorage(
+                [
+                    CloudFile(
+                        module["files"][i],
+                        [
+                            CloudFileVersion(
+                                module["files"][i],
+                                module["hashes"][i],
+                                int(module["filessize"][i]),
+                                module["filescrt"][i],
+                                module["filesmod"][i],
+                            )
+                        ],
+                    )
+                    for i in range(len(module["files"]))
+                ]
+            )
 
-            if "patches" in module:
-                num_patches = len(module["patches"])
+        if "monitoring" in module and module["monitoring"] != None:
+            for i in range(len(module["monitoring"])):
+                self._monitoring.append(
+                    (module["monitoring"][i], module["monitoringmod"][i])
+                )
 
-            if "notebook" in module:
-                program = module["notebook"]
+        if "patches" in module and module["patches"] != None:
+            self._num_patches = len(module["patches"])
 
-            if "machine" in module:
-                machine = Machine(*Machine.reconstruct(module["machine"]))
+        if "notebook" in module and module["notebook"] != None:
+            self._program = module["notebook"]
 
-            if "token" in module:
-                token = module["token"]
+        if "machine" in module and module["machine"] != None:
+            self._machine = Machine(*Machine.reconstruct(module["machine"]))
 
-        return (
-            app_map["name"],
-            program,
-            app_map["uuid"],
-            module_uuid,
-            json.loads(app_map["nbConfig"]),
-            app_map["runNum"],
-            initializing,
-            preparing,
-            running,
-            input_files,
-            updates,
-            output,
-            num_patches,
-            output_files,
-            machine,
-            token,
+        if "token" in module and module["token"] != None:
+            self._token = module["token"]
+
+        return output_to_print
+
+    @classmethod
+    def reconstruct(cls, workload):
+        w = Workload(
+            workload["name"],
+            None,
+            workload["uuid"],
+            workload["modules"][0]["uuid"] if len(workload["modules"]) > 0 else None,
+            workload["nbConfig"],
+            workload["runNum"],
         )
 
+        w._handle_workload_update(workload)
+
+        return w
+
     def _refresh(self):
         now = time.time()
         if now - self._last_refresh > 5:
             self._last_refresh = now
-            user_information = json.loads(optumi.core.get_user_information(True).text)
+            user_information = json.loads(optumi.core.get_workloads().text)
             # Add apps from user information if they don't already exist
             if "jobs" in user_information:
-                for app_map in user_information["jobs"]:
-                    if app_map["uuid"] == self._workload_uuid:
-                        (
-                            _,
-                            _,
-                            _,
-                            _,
-                            _,
-                            _,
-                            self._initializing_lines,
-                            self._preparing_lines,
-                            self._running_lines,
-                            self._input_files,
-                            self._update_lines,
-                            self._output,
-                            self._last_patches,
-                            self._output_files,
-                            self._machine,
-                            self._token,
-                        ) = Workload.reconstruct(app_map)
+                for workload in user_information["jobs"]:
+                    if workload["uuid"] == self._workload_uuid:
+                        self._handle_workload_update(workload)
 
     # def __print_status(self):
     #     collapsed = collapseUpdates(
@@ -236,7 +198,7 @@ class Workload:
     #     time.sleep(0.3)
 
     def __print_output(self, progress, output_to_print):
-        if progress == Progress.DETAIL:
+        if progress == "detail":
             for output in output_to_print:
                 print(output[0], end="")
 
@@ -248,7 +210,7 @@ class Workload:
                 and update[0] != "stop"
                 and update[0] != "error"
             ):
-                if progress == Progress.SUMMARY:
+                if progress == "summary":
                     print(
                         update[0]
                         if update[0].endswith("\n")
@@ -256,10 +218,10 @@ class Workload:
                         end="",
                         flush=True,
                     )
-                # elif progress == Progress.STATUS:
+                # elif progress == "status":
                 #     self.__print_status()
 
-    def wait(self, progress=Progress.SUMMARY):
+    def wait(self, progress="summary"):
         self._last_line_length = 0
 
         self.__print_output(progress, self._output)
@@ -278,25 +240,7 @@ class Workload:
                 ).text
             )[self._workload_uuid]
 
-            updates_to_print = []
-
-            new_preparing = updates["preparing"]
-            new_preparing_mod = updates["preparingmod"]
-            for i in range(len(new_preparing)):
-                update = new_preparing[i]
-                update_mod = new_preparing_mod[i]
-                self._preparing_lines.append((update, update_mod))
-                updates_to_print.append(self._preparing_lines[-1])
-
-            new_running = updates["running"]
-            new_running_mod = updates["runningmod"]
-            for i in range(len(new_running)):
-                update = new_running[i]
-                update_mod = new_running_mod[i]
-                self._running_lines.append((update, update_mod))
-                updates_to_print.append(self._running_lines[-1])
-
-            self.__print_updates(progress, updates_to_print)
+            self.__print_updates(progress, self._handle_workload_update(updates))
 
             if self._module_uuid:
                 updates = json.loads(
@@ -305,54 +249,18 @@ class Workload:
                         [self._module_uuid],
                         [len(self._update_lines)],
                         [len(self._output)],
-                        [0],
+                        [len(self._monitoring)],
                         [self._last_patches],
+                        progress == "detail",
                     ).content
                 )[self._module_uuid]
 
-                new_updates = updates["updates"]
-                new_updates_mod = updates["updatesmod"]
-                for i in range(len(new_updates)):
-                    self._update_lines.append((new_updates[i], new_updates_mod[i]))
+                token = self._token
 
-                output_to_print = []
+                self.__print_output(progress, self._handle_module_update(updates))
 
-                new_output = updates["output"]
-                new_output_mod = updates["outputmod"]
-                for i in range(len(new_output)):
-                    update = new_output[i]
-                    self._output.append((update, new_output_mod[i]))
-                    output_to_print.append(self._output[-1])
-
-                self.__print_output(progress, output_to_print)
-
-                if "patches" in updates:
-                    new_patches = updates["patches"]
-                    self._last_patches += len(new_patches)
-
-                self._output_files = CloudStorage(
-                    [
-                        CloudFile(
-                            updates["files"][i],
-                            [
-                                CloudFileVersion(
-                                    updates["files"][i],
-                                    updates["hashes"][i],
-                                    int(updates["filessize"][i]),
-                                    updates["filescrt"][i],
-                                    updates["filesmod"][i],
-                                )
-                            ],
-                        )
-                        for i in range(len(updates["files"]))
-                    ]
-                )
-
-                if "machine" in updates:
-                    self._machine = Machine(*Machine.reconstruct(updates["machine"]))
-
-                if self._token == None and "token" in updates:
-                    self._token = updates["token"]
+                # If we set the token, print the link for the user to access his session and stop waiting
+                if token == None and self._token != None:
                     print(
                         "Session available at: https://"
                         + self._machine.dns_name
@@ -361,12 +269,12 @@ class Workload:
                     )
                     return
 
-            if self.__get_status() == Status.COMPLETED:
+            if self.__get_status() == "completed":
                 break
 
             time.sleep(5)
 
-        # if progress == Progress.STATUS:
+        # if progress == "status":
         #     print()
 
     def stop(self, wait=True):
@@ -384,17 +292,17 @@ class Workload:
                 exit(0)
         else:
             print("Stopping workload " + self.name + "...")
-            if self.status != Status.COMPLETED:
+            if self.status != "completed":
                 optumi.core.stop_notebook(
                     self._workload_uuid, None
                 )  # Remove the second argument when we move to 3.14
-                if wait and self.status == Status.RUNNING:
-                    self.wait(progress=Progress.SILENT)
+                if wait and self.status == "running":
+                    self.wait(progress="silent")
             print("...completed")
 
     def remove(self, wait=True):
         # TODO: Add an override flag to stop workload before removing
-        if self.status != Status.COMPLETED:
+        if self.status != "completed":
             self.stop()
         print("Removing workload " + self.name + "...")
         optumi.core.teardown_notebook(
@@ -411,12 +319,12 @@ class Workload:
         if len(self._preparing_lines) == 0:
             for update in self._initializing_lines:
                 if Workload.__is_error(update):
-                    return Status.COMPLETED
+                    return "completed"
 
         if len(self._running_lines) == 0:
             for update in self._preparing_lines:
                 if Workload.__is_error(update) or Workload.__is_terminated(update):
-                    return Status.COMPLETED
+                    return "completed"
 
         if len(self._update_lines) > 0:
             running = True
@@ -424,17 +332,17 @@ class Workload:
                 if update[1] == "stop":
                     running = False
             if running:
-                return Status.RUNNING
+                return "running"
 
         for update in self._running_lines:
             if Workload.__is_stop(update):
-                return Status.COMPLETED
+                return "completed"
 
         for update in self._preparing_lines:
             if Workload.__is_stop(update):
-                return Status.RUNNING
+                return "running"
 
-        return Status.LAUNCHING
+        return "launching"
 
     def __adjust_message(self, message):
         # We will say a session is starting until we can connect to it
@@ -530,6 +438,15 @@ class Workload:
     @property
     def log(self):
         self._refresh()
+        if len(self._output) == 0:
+            for workload in json.loads(
+                optumi.core.get_workload_properties(
+                    self._workload_uuid, [], ["output"]
+                ).text
+            )["jobs"]:
+                self._handle_workload_update(
+                    workload
+                )  # There will only be one job returned and it will be the correct one
         return Log(self._path, self._output)
 
     @property
@@ -545,6 +462,15 @@ class Workload:
     @property
     def program(self):
         self._refresh()
+        if self._program == None:
+            for workload in json.loads(
+                optumi.core.get_workload_properties(
+                    self._workload_uuid, [], ["notebook"]
+                ).text
+            )["jobs"]:
+                self._handle_workload_update(
+                    workload
+                )  # There will only be one job returned and it will be the correct one
         return Program(self._path, self._run_num, self._program)
 
     @property
@@ -555,6 +481,15 @@ class Workload:
     @property
     def output_files(self):
         self._refresh()
+        if self._program == None:
+            for workload in json.loads(
+                optumi.core.get_workload_properties(
+                    self._workload_uuid, [], ["files"]
+                ).text
+            )["jobs"]:
+                self._handle_workload_update(
+                    workload
+                )  # There will only be one job returned and it will be the correct one
         return self._output_files
 
     @property
